-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
--
-- Copyright 2025 VLSI Solution Oy
-- SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
--
-- Licensed under the Solderpad Hardware License v 2.1 (the "License");
-- you may not use this file except in compliance with the License, or,
-- at your option, the Apache License version 2.0.
-- You may obtain a copy of the License at
--
--    https://solderpad.org/licenses/SHL-2.1/
--
-- Unless required by applicable law or agreed to in writing, any work
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
--                               RISCV core
--                                 V1.0
--
--
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

-- - - - - - - -
--  libraries
-- - - - - - - -
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.rv_pkg.all;

-- pragma translate_off
use std.textio.all;
library std_developerskit;
use std_developerskit.std_iopak.all;
-- pragma translate_on

entity rv_soc_tb is
end entity rv_soc_tb;


architecture rtl of rv_soc_tb is

  
  component rv_soc
    generic ( CLK_FREQ    : natural := 50000000;           -- default 50M
              UART_SPEED  : natural :=  1000000;           -- default 1M
              NR_GPIOS    : integer range 1 to 32 := 8 );
    port ( clk_i          : in  std_logic;
           xrst_i         : in  std_logic;                 -- syncr reset
           --
           xmem_awready_i  : in  std_logic;                -- xmem
           xmem_wready_i   : in  std_logic;
           xmem_bvalid_i   : in  std_logic;
           xmem_bresp_i    : in  slv_01_00;
           xmem_bid_i      : in  slv_03_00;
           xmem_arready_i  : in  std_logic;
           xmem_rvalid_i   : in  std_logic;
           xmem_rdata_i    : in  slv_31_00;
           xmem_rresp_i    : in  slv_01_00;
           xmem_rid_i      : in  slv_03_00;
           xmem_rlast_i    : in  std_logic;
           xmem_awvalid_o  : out std_logic;
           xmem_awaddr_o   : out slv_31_00;
           xmem_awid_o     : out slv_03_00;
           xmem_awlen_o    : out slv_07_00;
           xmem_awburst_o  : out slv_01_00;
           xmem_wvalid_o   : out std_logic;
           xmem_wdata_o    : out slv_31_00;
           xmem_wstrb_o    : out slv_03_00;
           xmem_wlast_o    : out std_logic;
           xmem_bready_o   : out std_logic;
           xmem_arvalid_o  : out std_logic;
           xmem_araddr_o   : out slv_31_00;
           xmem_arid_o     : out slv_03_00;
           xmem_arlen_o    : out slv_07_00;
           xmem_arburst_o  : out slv_01_00;
           xmem_rready_o   : out std_logic;
           --
           uart_txd_i      : in  std_logic;                -- io pins
           uart_rxd_o      : out std_logic;
           uart2_txd_i     : in  std_logic;
           uart2_rxd_o     : out std_logic;
           --
           spi_miso_i      : in  std_logic;
           spi_clk_o       : out std_logic;
           spi_cs_o        : out std_logic;
           spi_mosi_o      : out std_logic;
           --
           gpio_i          : in  slv_t(NR_GPIOS-1 downto 0);
           gpio_oena_o     : out slv_t(NR_GPIOS-1 downto 0);
           gpio_o          : out slv_t(NR_GPIOS-1 downto 0) );
  end component;


  component axi_sram
    generic ( ram_size_g    : natural := 16*1024; -- in 32bit words, NOT FOR MIF
              -- NOTE: if mif file is not given then :
              --    "mif_file_name" : init mem with boot code
              --    "-linear_data-" : linear data is used (0,1,2,3...)
              --    "-zero_data-"   : mem array is reset (all bits '0')
              --    "-one_data-"    : mem array is set   (all bits '1')
              --    "-unknown-"     : else all mem is 'U'
              mif_file_g  : string  := "-unknown-" ); -- edit if mif file used
    port ( clk_i          : in  std_logic;
           xrst_i         : in  std_logic; -- xreset
           --
           --  AXI slave if INPUTS (16)
           --
           axi_awvalid_i  : in  std_logic;
           axi_awaddr_i   : in  slv_31_00;
           axi_awid_i     : in  slv_03_00;
           axi_awlen_i    : in  slv_07_00;
           axi_awburst_i  : in  slv_01_00;
           axi_wvalid_i   : in  std_logic;
           axi_wdata_i    : in  slv_31_00;
           axi_wstrb_i    : in  slv_03_00;
           axi_wlast_i    : in  std_logic;
           axi_bready_i   : in  std_logic;
           axi_arvalid_i  : in  std_logic;
           axi_araddr_i   : in  slv_31_00;
           axi_arid_i     : in  slv_03_00;
           axi_arlen_i    : in  slv_07_00;
           axi_arburst_i  : in  slv_01_00;
           axi_rready_i   : in  std_logic;
           --
           --  AXI slave if OUTPUTS (11)
           --
           axi_awready_o  : out std_logic; 
           axi_wready_o   : out std_logic; 
           axi_bvalid_o   : out std_logic; 
           axi_bresp_o    : out slv_01_00; 
           axi_bid_o      : out slv_03_00; 
           axi_arready_o  : out std_logic; 
           axi_rvalid_o   : out std_logic; 
           axi_rdata_o    : out slv_31_00; 
           axi_rresp_o    : out slv_01_00; 
           axi_rid_o      : out slv_03_00; 
           axi_rlast_o    : out std_logic );
  end component;

  
  signal clk_i           :  std_logic := '0';
  signal xrst_i          :  std_logic;                     -- syncr reset

  signal xmem_awready_i  : std_logic;                      -- xmem AX if
  signal xmem_wready_i   : std_logic;
  signal xmem_bvalid_i   : std_logic;
  signal xmem_bresp_i    : slv_01_00;
  signal xmem_bid_i      : slv_03_00;
  signal xmem_arready_i  : std_logic;
  signal xmem_rvalid_i   : std_logic;
  signal xmem_rdata_i    : slv_31_00;
  signal xmem_rresp_i    : slv_01_00;
  signal xmem_rid_i      : slv_03_00;
  signal xmem_rlast_i    : std_logic;
  signal xmem_awvalid_o  : std_logic;
  signal xmem_awaddr_o   : slv_31_00;
  signal xmem_awid_o     : slv_03_00;
  signal xmem_awlen_o    : slv_07_00;
  signal xmem_awburst_o  : slv_01_00;
  signal xmem_wvalid_o   : std_logic;
  signal xmem_wdata_o    : slv_31_00;
  signal xmem_wstrb_o    : slv_03_00;
  signal xmem_wlast_o    : std_logic;
  signal xmem_bready_o   : std_logic;
  signal xmem_arvalid_o  : std_logic;
  signal xmem_araddr_o   : slv_31_00;
  signal xmem_arid_o     : slv_03_00;
  signal xmem_arlen_o    : slv_07_00;
  signal xmem_arburst_o  : slv_01_00;
  signal xmem_rready_o   : std_logic;
  --
  signal uart_txd_i      : std_logic;                       -- io pins
  signal uart_rxd_o      : std_logic;
  signal uart2_txd_i     : std_logic;
  signal uart2_rxd_o     : std_logic;
  --
  signal spi_miso_i      : std_logic;
  signal spi_clk_o       : std_logic;
  signal spi_cs_o        : std_logic;
  signal spi_mosi_o      : std_logic;
  --
  signal gpio_i          : slv_t(8-1 downto 0);
  signal gpio_oena_o     : slv_t(8-1 downto 0);
  signal gpio_o          : slv_t(8-1 downto 0);


begin


  clk_i   <= not clk_i after 20 ns / 2;
  xrst_i  <= '0', '1' after 73 ns;                         -- syncr reset


  u_rv_soc: rv_soc
    generic map ( CLK_FREQ    => 50000000,                 -- default 50M
                  UART_SPEED  =>  1000000,                 -- default 1M
                  NR_GPIOS    => 8 )
  port map ( clk_i           => clk_i,
             xrst_i          => xrst_i,                    -- syncr reset
             xmem_awready_i  => xmem_awready_i,            -- xmem
             xmem_wready_i   => xmem_wready_i,
             xmem_bvalid_i   => xmem_bvalid_i,
             xmem_bresp_i    => xmem_bresp_i,
             xmem_bid_i      => xmem_bid_i,
             xmem_arready_i  => xmem_arready_i,
             xmem_rvalid_i   => xmem_rvalid_i,
             xmem_rdata_i    => xmem_rdata_i,
             xmem_rresp_i    => xmem_rresp_i,
             xmem_rid_i      => xmem_rid_i,
             xmem_rlast_i    => xmem_rlast_i,
             xmem_awvalid_o  => xmem_awvalid_o,
             xmem_awaddr_o   => xmem_awaddr_o,
             xmem_awid_o     => xmem_awid_o,
             xmem_awlen_o    => xmem_awlen_o,
             xmem_awburst_o  => xmem_awburst_o,
             xmem_wvalid_o   => xmem_wvalid_o,
             xmem_wdata_o    => xmem_wdata_o,
             xmem_wstrb_o    => xmem_wstrb_o,
             xmem_wlast_o    => xmem_wlast_o,
             xmem_bready_o   => xmem_bready_o,
             xmem_arvalid_o  => xmem_arvalid_o,
             xmem_araddr_o   => xmem_araddr_o,
             xmem_arid_o     => xmem_arid_o,
             xmem_arlen_o    => xmem_arlen_o,
             xmem_arburst_o  => xmem_arburst_o,
             xmem_rready_o   => xmem_rready_o,
             --
             uart_txd_i      => uart_txd_i,                -- io pins
             uart_rxd_o      => uart_rxd_o,
             uart2_txd_i     => uart2_txd_i,
             uart2_rxd_o     => uart2_rxd_o,
             --
             spi_miso_i      => spi_miso_i,
             spi_clk_o       => spi_clk_o,
             spi_cs_o        => spi_cs_o,
             spi_mosi_o      => spi_mosi_o,
             --
             gpio_i          => gpio_i,
             gpio_oena_o     => gpio_oena_o,
             gpio_o          => gpio_o );

  --
  uart_txd_i     <= 'H';                                   -- io pins
--uart_rxd_o
  uart2_txd_i    <= 'H';
--uart2_rxd_o
  --
  spi_miso_i     <= 'L';
--spi_clk_o
--spi_cs_o
--spi_mosi_o
  --
  gpio_i         <= "LLLLLLLL";
--gpio_oena_o    -- 7:0
--gpio_o         -- 7:0


  u_axi_sram : axi_sram
    generic map ( ram_size_g   => 16*1024,   -- in 32bit words, NOT FOR MIF
                  -- NOTE: if mif file is not given then :
                  --    "mif_file_name" : init mem with boot code
                  --    "-linear_data-" : linear data is used (0,1,2,3...)
                  --    "-zero_data-"   : mem array is reset (all bits '0')
                  --    "-one_data-"    : mem array is set   (all bits '1')
                  --    "-unknown-"     : else all mem is 'U'
                  --
                  -- Linux 4.20 kerner image,  edit if other mif file used
                  mif_file_g    => "../rtl/tb/linux_riscv_soc.elf.mif" )
    port map ( clk_i            => clk_i,
               xrst_i           => xrst_i,
               --
               -- AXI slave if INPUTS (16)
               --
               axi_awvalid_i    => xmem_awvalid_o,
               axi_awaddr_i     => xmem_awaddr_o,
               axi_awid_i       => xmem_awid_o,
               axi_awlen_i      => xmem_awlen_o,
               axi_awburst_i    => xmem_awburst_o,
               axi_wvalid_i     => xmem_wvalid_o,
               axi_wdata_i      => xmem_wdata_o,
               axi_wstrb_i      => xmem_wstrb_o,
               axi_wlast_i      => xmem_wlast_o,
               axi_bready_i     => xmem_bready_o,
               axi_arvalid_i    => xmem_arvalid_o,
               axi_araddr_i     => xmem_araddr_o,
               axi_arid_i       => xmem_arid_o,
               axi_arlen_i      => xmem_arlen_o,
               axi_arburst_i    => xmem_arburst_o,
               axi_rready_i     => xmem_rready_o,
               --
               -- AXI slave if OUTPUTS (11)
               --
               axi_awready_o    => xmem_awready_i,
               axi_wready_o     => xmem_wready_i,
               axi_bvalid_o     => xmem_bvalid_i,
               axi_bresp_o      => xmem_bresp_i,
               axi_bid_o        => xmem_bid_i,
               axi_arready_o    => xmem_arready_i,
               axi_rvalid_o     => xmem_rvalid_i,
               axi_rdata_o      => xmem_rdata_i,
               axi_rresp_o      => xmem_rresp_i,
               axi_rid_o        => xmem_rid_i,
               axi_rlast_o      => xmem_rlast_i );

end rtl;
