-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
--
-- Copyright 2025 VLSI Solution Oy
-- SPDX-License-Identifier: Apache-2.0 WITH SHL-2.1
--
-- Licensed under the Solderpad Hardware License v 2.1 (the "License");
-- you may not use this file except in compliance with the License, or,
-- at your option, the Apache License version 2.0.
-- You may obtain a copy of the License at
--
--    https://solderpad.org/licenses/SHL-2.1/
--
-- Unless required by applicable law or agreed to in writing, any work
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
--
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
--                               RISCV core
--                                 V1.0
--
--
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

-- - - - - - - -
--  libraries
-- - - - - - - -
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.rv_pkg.all;


entity axi_mem_if is
  port ( clk_i             : in  std_logic;
         xrst_i            : in  std_logic;                 -- syncr reset
         --
         inport_awvalid_i  : in  std_logic;                 -- dbg uart
         inport_awaddr_i   : in  slv_31_00;
         inport_awid_i     : in  slv_03_00;                 --    X"0"
         inport_awlen_i    : in  slv_07_00;                 --    X"00";
         inport_awburst_i  : in  slv_01_00;                 --    "01"
         inport_wvalid_i   : in  std_logic;
         inport_wdata_i    : in  slv_31_00;
         inport_wstrb_i    : in  slv_03_00;
         inport_wlast_i    : in  std_logic;                 --    '1'
         inport_bready_i   : in  std_logic;
         inport_arvalid_i  : in  std_logic;
         inport_araddr_i   : in  slv_31_00;
         inport_arid_i     : in  slv_03_00;                 --    X"0"
         inport_arlen_i    : in  slv_07_00;                 --    X"00"
         inport_arburst_i  : in  slv_01_00;                 --    "01"
         inport_rready_i   : in  std_logic;
         inport_awready_o  : out std_logic;
         inport_wready_o   : out std_logic;
         inport_bvalid_o   : out std_logic;
         inport_bresp_o    : out slv_01_00;
         inport_bid_o      : out slv_03_00;
         inport_arready_o  : out std_logic;
         inport_rvalid_o   : out std_logic;
         inport_rdata_o    : out slv_31_00;
         inport_rresp_o    : out slv_01_00;
         inport_rid_o      : out slv_03_00;
         inport_rlast_o    : out std_logic;
         --
         cpu_i_awvalid_i   : in  std_logic;                 -- i-cache
         cpu_i_awaddr_i    : in  slv_31_00;
         cpu_i_awid_i      : in  slv_03_00;
         cpu_i_awlen_i     : in  slv_07_00;
         cpu_i_awburst_i   : in  slv_01_00;
         cpu_i_wvalid_i    : in  std_logic;
         cpu_i_wdata_i     : in  slv_31_00;
         cpu_i_wstrb_i     : in  slv_03_00;
         cpu_i_wlast_i     : in  std_logic;
         cpu_i_bready_i    : in  std_logic;
         cpu_i_arvalid_i   : in  std_logic;
         cpu_i_araddr_i    : in  slv_31_00;
         cpu_i_arid_i      : in  slv_03_00;
         cpu_i_arlen_i     : in  slv_07_00;
         cpu_i_arburst_i   : in  slv_01_00;
         cpu_i_rready_i    : in  std_logic;
         cpu_i_awready_o   : out std_logic;
         cpu_i_wready_o    : out std_logic;
         cpu_i_bvalid_o    : out std_logic;
         cpu_i_bresp_o     : out slv_01_00;
         cpu_i_bid_o       : out slv_03_00;
         cpu_i_arready_o   : out std_logic;
         cpu_i_rvalid_o    : out std_logic;
         cpu_i_rdata_o     : out slv_31_00;
         cpu_i_rresp_o     : out slv_01_00;
         cpu_i_rid_o       : out slv_03_00;
         cpu_i_rlast_o     : out std_logic;
         --
         cpu_d_awready_o   : out std_logic;                 -- d-cache
         cpu_d_wready_o    : out std_logic;
         cpu_d_bvalid_o    : out std_logic;
         cpu_d_bresp_o     : out slv_01_00;
         cpu_d_bid_o       : out slv_03_00;
         cpu_d_arready_o   : out std_logic;
         cpu_d_rvalid_o    : out std_logic;
         cpu_d_rdata_o     : out slv_31_00;
         cpu_d_rresp_o     : out slv_01_00;
         cpu_d_rid_o       : out slv_03_00;
         cpu_d_rlast_o     : out std_logic;
         cpu_d_awvalid_i   : in  std_logic;
         cpu_d_awaddr_i    : in  slv_31_00;
         cpu_d_awid_i      : in  slv_03_00;
         cpu_d_awlen_i     : in  slv_07_00;
         cpu_d_awburst_i   : in  slv_01_00;
         cpu_d_wvalid_i    : in  std_logic;
         cpu_d_wdata_i     : in  slv_31_00;
         cpu_d_wstrb_i     : in  slv_03_00;
         cpu_d_wlast_i     : in  std_logic;
         cpu_d_bready_i    : in  std_logic;
         cpu_d_arvalid_i   : in  std_logic;
         cpu_d_araddr_i    : in  slv_31_00;
         cpu_d_arid_i      : in  slv_03_00;
         cpu_d_arlen_i     : in  slv_07_00;
         cpu_d_arburst_i   : in  slv_01_00;
         cpu_d_rready_i    : in  std_logic;
         --
         mem_awready_i     : in  std_logic;                 -- xmem
         mem_wready_i      : in  std_logic;
         mem_bvalid_i      : in  std_logic;
         mem_bresp_i       : in  slv_01_00;
         mem_bid_i         : in  slv_03_00;                 -- NC
         mem_arready_i     : in  std_logic;
         mem_rvalid_i      : in  std_logic;
         mem_rdata_i       : in  slv_31_00;
         mem_rresp_i       : in  slv_01_00;
         mem_rid_i         : in  slv_03_00;                 -- NC
         mem_rlast_i       : in  std_logic;                 -- NC
         mem_awvalid_o     : out std_logic;
         mem_awaddr_o      : out slv_31_00;
         mem_awid_o        : out slv_03_00;
         mem_awlen_o       : out slv_07_00;
         mem_awburst_o     : out slv_01_00;
         mem_wvalid_o      : out std_logic;
         mem_wdata_o       : out slv_31_00;
         mem_wstrb_o       : out slv_03_00;
         mem_wlast_o       : out std_logic;
         mem_bready_o      : out std_logic;
         mem_arvalid_o     : out std_logic;
         mem_araddr_o      : out slv_31_00;
         mem_arid_o        : out slv_03_00;
         mem_arlen_o       : out slv_07_00;
         mem_arburst_o     : out slv_01_00;
         mem_rready_o      : out std_logic
         );
end entity axi_mem_if;


architecture rtl of axi_mem_if is

  component axi4_arb
    port ( clk_i              : in  std_logic;
           xrst_i             : in  std_logic;
           --
           inport_awvalid_i   : in  std_logic;              -- dbg uart
           inport_awaddr_i    : in  slv_31_00;
           inport_awid_i      : in  slv_03_00;
           inport_awlen_i     : in  slv_07_00;
           inport_awburst_i   : in  slv_01_00;
           inport_wvalid_i    : in  std_logic;
           inport_wdata_i     : in  slv_31_00;
           inport_wstrb_i     : in  slv_03_00;
           inport_wlast_i     : in  std_logic;
           inport_bready_i    : in  std_logic;
           inport_arvalid_i   : in  std_logic;
           inport_araddr_i    : in  slv_31_00;
           inport_arid_i      : in  slv_03_00;
           inport_arlen_i     : in  slv_07_00;
           inport_arburst_i   : in  slv_01_00;
           inport_rready_i    : in  std_logic;
           inport_awready_o   : out std_logic;
           inport_wready_o    : out std_logic;
           inport_bvalid_o    : out std_logic;
           inport_bresp_o     : out slv_01_00;
           inport_bid_o       : out slv_03_00;
           inport_arready_o   : out std_logic;
           inport_rvalid_o    : out std_logic;
           inport_rdata_o     : out slv_31_00;
           inport_rresp_o     : out slv_01_00;
           inport_rid_o       : out slv_03_00;
           inport_rlast_o     : out std_logic;
           --
           cpu_d_awvalid_i    : in  std_logic;              -- d-cache
           cpu_d_awaddr_i     : in  slv_31_00;
           cpu_d_awid_i       : in  slv_03_00;
           cpu_d_awlen_i      : in  slv_07_00;
           cpu_d_awburst_i    : in  slv_01_00;
           cpu_d_wvalid_i     : in  std_logic;
           cpu_d_wdata_i      : in  slv_31_00;
           cpu_d_wstrb_i      : in  slv_03_00;
           cpu_d_wlast_i      : in  std_logic;
           cpu_d_bready_i     : in  std_logic;
           cpu_d_arvalid_i    : in  std_logic;
           cpu_d_araddr_i     : in  slv_31_00;
           cpu_d_arid_i       : in  slv_03_00;
           cpu_d_arlen_i      : in  slv_07_00;
           cpu_d_arburst_i    : in  slv_01_00;
           cpu_d_rready_i     : in  std_logic;
           cpu_d_awready_o    : out std_logic;
           cpu_d_wready_o     : out std_logic;
           cpu_d_bvalid_o     : out std_logic;
           cpu_d_bresp_o      : out slv_01_00;
           cpu_d_bid_o        : out slv_03_00;
           cpu_d_arready_o    : out std_logic;
           cpu_d_rvalid_o     : out std_logic;
           cpu_d_rdata_o      : out slv_31_00;
           cpu_d_rresp_o      : out slv_01_00;
           cpu_d_rid_o        : out slv_03_00;
           cpu_d_rlast_o      : out std_logic;
           --
           cpu_i_awvalid_i    : in  std_logic;              -- i-cache
           cpu_i_awaddr_i     : in  slv_31_00;
           cpu_i_awid_i       : in  slv_03_00;
           cpu_i_awlen_i      : in  slv_07_00;
           cpu_i_awburst_i    : in  slv_01_00;
           cpu_i_wvalid_i     : in  std_logic;
           cpu_i_wdata_i      : in  slv_31_00;
           cpu_i_wstrb_i      : in  slv_03_00;
           cpu_i_wlast_i      : in  std_logic;
           cpu_i_bready_i     : in  std_logic;
           cpu_i_arvalid_i    : in  std_logic;
           cpu_i_araddr_i     : in  slv_31_00;
           cpu_i_arid_i       : in  slv_03_00;
           cpu_i_arlen_i      : in  slv_07_00;
           cpu_i_arburst_i    : in  slv_01_00;
           cpu_i_rready_i     : in  std_logic;
           cpu_i_awready_o    : out std_logic;
           cpu_i_wready_o     : out std_logic;
           cpu_i_bvalid_o     : out std_logic;
           cpu_i_bresp_o      : out slv_01_00;
           cpu_i_bid_o        : out slv_03_00;
           cpu_i_arready_o    : out std_logic;
           cpu_i_rvalid_o     : out std_logic;
           cpu_i_rdata_o      : out slv_31_00;
           cpu_i_rresp_o      : out slv_01_00;
           cpu_i_rid_o        : out slv_03_00;
           cpu_i_rlast_o      : out std_logic;
           --
           outport_awready_i  : in  std_logic;              -- xmem
           outport_wready_i   : in  std_logic;
           outport_bvalid_i   : in  std_logic;
           outport_bresp_i    : in  slv_01_00;
           outport_bid_i      : in  slv_03_00;
           outport_arready_i  : in  std_logic;
           outport_rvalid_i   : in  std_logic;
           outport_rdata_i    : in  slv_31_00;
           outport_rresp_i    : in  slv_01_00;
           outport_rid_i      : in  slv_03_00;
           outport_rlast_i    : in  std_logic;
           outport_awvalid_o  : out std_logic;
           outport_awaddr_o   : out slv_31_00;
           outport_awid_o     : out slv_03_00;
           outport_awlen_o    : out slv_07_00;
           outport_awburst_o  : out slv_01_00;
           outport_wvalid_o   : out std_logic;
           outport_wdata_o    : out slv_31_00;
           outport_wstrb_o    : out slv_03_00;
           outport_wlast_o    : out std_logic;
           outport_bready_o   : out std_logic;
           outport_arvalid_o  : out std_logic;
           outport_araddr_o   : out slv_31_00;
           outport_arid_o     : out slv_03_00;
           outport_arlen_o    : out slv_07_00;
           outport_arburst_o  : out slv_01_00;
           outport_rready_o   : out std_logic );
  end component;


  component axi4_retime
    port ( clk_i              : in  std_logic;
           xrst_i             : in  std_logic;
           --
           inport_awvalid_i   : in  std_logic;
           inport_awaddr_i    : in  slv_31_00;
           inport_awid_i      : in  slv_03_00;
           inport_awlen_i     : in  slv_07_00;
           inport_awburst_i   : in  slv_01_00;
           inport_wvalid_i    : in  std_logic;
           inport_wdata_i     : in  slv_31_00;
           inport_wstrb_i     : in  slv_03_00;
           inport_wlast_i     : in  std_logic;
           inport_bready_i    : in  std_logic;
           inport_arvalid_i   : in  std_logic;
           inport_araddr_i    : in  slv_31_00;
           inport_arid_i      : in  slv_03_00;
           inport_arlen_i     : in  slv_07_00;
           inport_arburst_i   : in  slv_01_00;
           inport_rready_i    : in  std_logic;
           inport_awready_o   : out std_logic;
           inport_wready_o    : out std_logic;
           inport_bvalid_o    : out std_logic;
           inport_bresp_o     : out slv_01_00;
           inport_bid_o       : out slv_03_00;
           inport_arready_o   : out std_logic;
           inport_rvalid_o    : out std_logic;
           inport_rdata_o     : out slv_31_00;
           inport_rresp_o     : out slv_01_00;
           inport_rid_o       : out slv_03_00;
           inport_rlast_o     : out std_logic;
           --
           outport_awready_i  : in  std_logic;
           outport_wready_i   : in  std_logic;
           outport_bvalid_i   : in  std_logic;
           outport_bresp_i    : in  slv_01_00;
           outport_bid_i      : in  slv_03_00;
           outport_arready_i  : in  std_logic;
           outport_rvalid_i   : in  std_logic;
           outport_rdata_i    : in  slv_31_00;
           outport_rresp_i    : in  slv_01_00;
           outport_rid_i      : in  slv_03_00;
           outport_rlast_i    : in  std_logic;
           outport_awvalid_o  : out std_logic;
           outport_awaddr_o   : out slv_31_00;
           outport_awid_o     : out slv_03_00;
           outport_awlen_o    : out slv_07_00;
           outport_awburst_o  : out slv_01_00;
           outport_wvalid_o   : out std_logic;
           outport_wdata_o    : out slv_31_00;
           outport_wstrb_o    : out slv_03_00;
           outport_wlast_o    : out std_logic;
           outport_bready_o   : out std_logic;
           outport_arvalid_o  : out std_logic;
           outport_araddr_o   : out slv_31_00;
           outport_arid_o     : out slv_03_00;
           outport_arlen_o    : out slv_07_00;
           outport_arburst_o  : out slv_01_00;
           outport_rready_o   : out std_logic );
  end component;


  signal axi_arb_arvalid_w  : std_logic;
  signal axi_arb_rready_w   : std_logic;
  signal axi_arb_wvalid_w   : std_logic;
  signal axi_arb_arid_w     : slv_03_00;
  signal axi_arb_bid_w      : slv_03_00;
  signal axi_arb_awready_w  : std_logic;
  signal axi_arb_wready_w   : std_logic;
  signal axi_arb_bready_w   : std_logic;
  signal axi_arb_wlast_w    : std_logic;
  signal axi_arb_awlen_w    : slv_07_00;
  signal axi_arb_awburst_w  : slv_01_00;
  signal axi_arb_rid_w      : slv_03_00;
  signal axi_arb_bresp_w    : slv_01_00;
  signal axi_arb_araddr_w   : slv_31_00;
  signal axi_arb_rresp_w    : slv_01_00;
  signal axi_arb_awaddr_w   : slv_31_00;
  signal axi_arb_rvalid_w   : std_logic;
  signal axi_arb_wstrb_w    : slv_03_00;
  signal axi_arb_wdata_w    : slv_31_00;
  signal axi_arb_rlast_w    : std_logic;
  signal axi_arb_arburst_w  : slv_01_00;
  signal axi_arb_bvalid_w   : std_logic;
  signal axi_arb_arready_w  : std_logic;
  signal axi_arb_rdata_w    : slv_31_00;
  signal axi_arb_awvalid_w  : std_logic;
  signal axi_arb_awid_w     : slv_03_00;
  signal axi_arb_arlen_w    : slv_07_00;

begin

  u_arb : axi4_arb
    port map ( clk_i              => clk_i,
               xrst_i             => xrst_i,
               --
               inport_awvalid_i   => inport_awvalid_i,      -- dbg uart
               inport_awaddr_i    => inport_awaddr_i,
               inport_awid_i      => inport_awid_i,         --    X"0"
               inport_awlen_i     => inport_awlen_i,        --    X"00"
               inport_awburst_i   => inport_awburst_i,      --    "01"
               inport_wvalid_i    => inport_wvalid_i,
               inport_wdata_i     => inport_wdata_i,
               inport_wstrb_i     => inport_wstrb_i,
               inport_wlast_i     => inport_wlast_i,        --    '1'
               inport_bready_i    => inport_bready_i,
               inport_arvalid_i   => inport_arvalid_i,
               inport_araddr_i    => inport_araddr_i,
               inport_arid_i      => inport_arid_i,         --    X"0"
               inport_arlen_i     => inport_arlen_i,        --    X"00"
               inport_arburst_i   => inport_arburst_i,      --    "01"
               inport_rready_i    => inport_rready_i,
               inport_awready_o   => inport_awready_o,
               inport_wready_o    => inport_wready_o,
               inport_bvalid_o    => inport_bvalid_o,
               inport_bresp_o     => inport_bresp_o,
               inport_bid_o       => inport_bid_o,
               inport_arready_o   => inport_arready_o,
               inport_rvalid_o    => inport_rvalid_o,
               inport_rdata_o     => inport_rdata_o,
               inport_rresp_o     => inport_rresp_o,
               inport_rid_o       => inport_rid_o,
               inport_rlast_o     => inport_rlast_o,
                --
               cpu_d_awvalid_i    => cpu_d_awvalid_i,       -- d-cache
               cpu_d_awaddr_i     => cpu_d_awaddr_i,
               cpu_d_awid_i       => cpu_d_awid_i,
               cpu_d_awlen_i      => cpu_d_awlen_i,
               cpu_d_awburst_i    => cpu_d_awburst_i,
               cpu_d_wvalid_i     => cpu_d_wvalid_i,
               cpu_d_wdata_i      => cpu_d_wdata_i,
               cpu_d_wstrb_i      => cpu_d_wstrb_i,
               cpu_d_wlast_i      => cpu_d_wlast_i,
               cpu_d_bready_i     => cpu_d_bready_i,
               cpu_d_arvalid_i    => cpu_d_arvalid_i,
               cpu_d_araddr_i     => cpu_d_araddr_i,
               cpu_d_arid_i       => cpu_d_arid_i,
               cpu_d_arlen_i      => cpu_d_arlen_i,
               cpu_d_arburst_i    => cpu_d_arburst_i,
               cpu_d_rready_i     => cpu_d_rready_i,
               cpu_d_awready_o    => cpu_d_awready_o,
               cpu_d_wready_o     => cpu_d_wready_o,
               cpu_d_bvalid_o     => cpu_d_bvalid_o,
               cpu_d_bresp_o      => cpu_d_bresp_o,
               cpu_d_bid_o        => cpu_d_bid_o,
               cpu_d_arready_o    => cpu_d_arready_o,
               cpu_d_rvalid_o     => cpu_d_rvalid_o,
               cpu_d_rdata_o      => cpu_d_rdata_o,
               cpu_d_rresp_o      => cpu_d_rresp_o,
               cpu_d_rid_o        => cpu_d_rid_o,
               cpu_d_rlast_o      => cpu_d_rlast_o,
               --
               cpu_i_awvalid_i    => cpu_i_awvalid_i,       -- i-cache
               cpu_i_awaddr_i     => cpu_i_awaddr_i,
               cpu_i_awid_i       => cpu_i_awid_i,
               cpu_i_awlen_i      => cpu_i_awlen_i,
               cpu_i_awburst_i    => cpu_i_awburst_i,
               cpu_i_wvalid_i     => cpu_i_wvalid_i,
               cpu_i_wdata_i      => cpu_i_wdata_i,
               cpu_i_wstrb_i      => cpu_i_wstrb_i,
               cpu_i_wlast_i      => cpu_i_wlast_i,
               cpu_i_bready_i     => cpu_i_bready_i,
               cpu_i_arvalid_i    => cpu_i_arvalid_i,
               cpu_i_araddr_i     => cpu_i_araddr_i,
               cpu_i_arid_i       => cpu_i_arid_i,
               cpu_i_arlen_i      => cpu_i_arlen_i,
               cpu_i_arburst_i    => cpu_i_arburst_i,
               cpu_i_rready_i     => cpu_i_rready_i,
               cpu_i_awready_o    => cpu_i_awready_o,
               cpu_i_wready_o     => cpu_i_wready_o,
               cpu_i_bvalid_o     => cpu_i_bvalid_o,
               cpu_i_bresp_o      => cpu_i_bresp_o,
               cpu_i_bid_o        => cpu_i_bid_o,
               cpu_i_arready_o    => cpu_i_arready_o,
               cpu_i_rvalid_o     => cpu_i_rvalid_o,
               cpu_i_rdata_o      => cpu_i_rdata_o,
               cpu_i_rresp_o      => cpu_i_rresp_o,
               cpu_i_rid_o        => cpu_i_rid_o,
               cpu_i_rlast_o      => cpu_i_rlast_o,
               --
               outport_awready_i  => axi_arb_awready_w,     -- xmem
               outport_wready_i   => axi_arb_wready_w,
               outport_bvalid_i   => axi_arb_bvalid_w,
               outport_bresp_i    => axi_arb_bresp_w,
               outport_bid_i      => axi_arb_bid_w,
               outport_arready_i  => axi_arb_arready_w,
               outport_rvalid_i   => axi_arb_rvalid_w,
               outport_rdata_i    => axi_arb_rdata_w,
               outport_rresp_i    => axi_arb_rresp_w,
               outport_rid_i      => axi_arb_rid_w,
               outport_rlast_i    => axi_arb_rlast_w,
               outport_awvalid_o  => axi_arb_awvalid_w,
               outport_awaddr_o   => axi_arb_awaddr_w,
               outport_awid_o     => axi_arb_awid_w,
               outport_awlen_o    => axi_arb_awlen_w,
               outport_awburst_o  => axi_arb_awburst_w,
               outport_wvalid_o   => axi_arb_wvalid_w,
               outport_wdata_o    => axi_arb_wdata_w,
               outport_wstrb_o    => axi_arb_wstrb_w,
               outport_wlast_o    => axi_arb_wlast_w,
               outport_bready_o   => axi_arb_bready_w,
               outport_arvalid_o  => axi_arb_arvalid_w,
               outport_araddr_o   => axi_arb_araddr_w,
               outport_arid_o     => axi_arb_arid_w,
               outport_arlen_o    => axi_arb_arlen_w,
               outport_arburst_o  => axi_arb_arburst_w,
               outport_rready_o   => axi_arb_rready_w );


  u_retime : axi4_retime
    port map ( clk_i              => clk_i,
               xrst_i             => xrst_i,
               --
               inport_awvalid_i   => axi_arb_awvalid_w,
               inport_awaddr_i    => axi_arb_awaddr_w,
               inport_awid_i      => axi_arb_awid_w,
               inport_awlen_i     => axi_arb_awlen_w,
               inport_awburst_i   => axi_arb_awburst_w,
               inport_wvalid_i    => axi_arb_wvalid_w,
               inport_wdata_i     => axi_arb_wdata_w,
               inport_wstrb_i     => axi_arb_wstrb_w,
               inport_wlast_i     => axi_arb_wlast_w,
               inport_bready_i    => axi_arb_bready_w,
               inport_arvalid_i   => axi_arb_arvalid_w,
               inport_araddr_i    => axi_arb_araddr_w,
               inport_arid_i      => axi_arb_arid_w,
               inport_arlen_i     => axi_arb_arlen_w,
               inport_arburst_i   => axi_arb_arburst_w,
               inport_rready_i    => axi_arb_rready_w,
               inport_awready_o   => axi_arb_awready_w,
               inport_wready_o    => axi_arb_wready_w,
               inport_bvalid_o    => axi_arb_bvalid_w,
               inport_bresp_o     => axi_arb_bresp_w,
               inport_bid_o       => axi_arb_bid_w,
               inport_arready_o   => axi_arb_arready_w,
               inport_rvalid_o    => axi_arb_rvalid_w,
               inport_rdata_o     => axi_arb_rdata_w,
               inport_rresp_o     => axi_arb_rresp_w,
               inport_rid_o       => axi_arb_rid_w,
               inport_rlast_o     => axi_arb_rlast_w,
               --
               outport_awready_i  => mem_awready_i,         -- from DDR
               outport_wready_i   => mem_wready_i,          -- from DDR
               outport_bvalid_i   => mem_bvalid_i,          -- from DDR
               outport_bresp_i    => mem_bresp_i,           -- from DDR [1:0]
               outport_bid_i      => mem_bid_i,             -- from DDR [3:0]
               outport_arready_i  => mem_arready_i,         -- from DDR
               outport_rvalid_i   => mem_rvalid_i,          -- from DDR
               outport_rdata_i    => mem_rdata_i,           -- from DDR [31:0]
               outport_rresp_i    => mem_rresp_i,           -- from DDR [1:0]
               outport_rid_i      => mem_rid_i,             -- from DDR [3:0]
               outport_rlast_i    => mem_rlast_i,           -- from DDR
               --
               outport_awvalid_o  => mem_awvalid_o,         -- to DDR
               outport_awaddr_o   => mem_awaddr_o,          -- to DDR [31:0]
               outport_awid_o     => mem_awid_o,            -- to DDR [3:0]
               outport_awlen_o    => mem_awlen_o,           -- to DDR [7:0]
               outport_awburst_o  => mem_awburst_o,         -- to DDR [1:0]
               outport_wvalid_o   => mem_wvalid_o,          -- to DDR
               outport_wdata_o    => mem_wdata_o,           -- to DDR [31:0]
               outport_wstrb_o    => mem_wstrb_o,           -- to DDR [3:0]
               outport_wlast_o    => mem_wlast_o,           -- to DDR
               outport_bready_o   => mem_bready_o,          -- to DDR
               outport_arvalid_o  => mem_arvalid_o,         -- to DDR
               outport_araddr_o   => mem_araddr_o,          -- to DDR [31:0]
               outport_arid_o     => mem_arid_o,            -- to DDR [3:0]
               outport_arlen_o    => mem_arlen_o,           -- to DDR [7:0]
               outport_arburst_o  => mem_arburst_o,         -- to DDR [1:0]
               outport_rready_o   => mem_rready_o );        -- to DDR

end rtl;
