

radix -hexadecimal

  #
  # Send one byte throug uart
  #
  #   50MHz clock, 1M uart 
  #     50 clks * 20ns (ONE BIT TIME IN NANOSECONDS)
  proc SEND_BYTE { ubyte { bittime {1000} }  } {
     global Now
     set bitdur $bittime ;
     # start bit "0"
     force -freeze /rv_soc_tcm/uart_txd_i 0 0
     run   $bitdur ns
     for {set bb 0} {$bb < 8} {incr bb} {
       set rxbit   [expr ($ubyte>>$bb) & 0x1]
       force -freeze /rv_soc_tcm/uart_txd_i $rxbit 0
       run   $bitdur ns
     }
     # stop bit "1"
     force -freeze /rv_soc_tcm/uart_txd_i 1 0
     run   $bitdur ns
     # release RX (there is a weak pull up in testbench)
     noforce /rv_soc_tcm/uart_txd_i
  }


  #
  #  REQ_WRITE = X"10",  LEN, ADDR3,2,1,0,  DATA0,1,2,3
  #
  proc REQ_WRITE { len8 addr32 data32 } {
    #    CMD: wr = 0x10
    SEND_BYTE 0x10
    #    LEN (bytes)
    SEND_BYTE $len8
    #    ADDR (32 bits)                       ! MSB 1st
    SEND_BYTE [expr ($addr32>>24) & 0xFF]
    SEND_BYTE [expr ($addr32>>16) & 0xFF]
    SEND_BYTE [expr ($addr32>>8)  & 0xFF]
    SEND_BYTE [expr  $addr32      & 0xFF]
    #    DATA (LEN x bytes)                   ! LSB 1st
    SEND_BYTE [expr  $data32      & 0xFF]
    SEND_BYTE [expr ($data32>>8)  & 0xFF]
    SEND_BYTE [expr ($data32>>16) & 0xFF]
    SEND_BYTE [expr ($data32>>24) & 0xFF]
  }


  #
  #  REQ_READ = X"11",  LEN, ADDR3,2,1,0,  DATA0,1,2,3
  #
  proc REQ_READ { len8 addr32 } {
    #    CMD: wr = 0x10
    SEND_BYTE 0x11
    #    LEN (bytes)
    SEND_BYTE $len8
    #    ADDR (32 bits)                       ! MSB 1st
    SEND_BYTE [expr ($addr32>>24) & 0xFF]
    SEND_BYTE [expr ($addr32>>16) & 0xFF]
    SEND_BYTE [expr ($addr32>>8)  & 0xFF]
    SEND_BYTE [expr  $addr32      & 0xFF]
    run 60 us ;
  }


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  add wave /rv_soc_tcm/u_rv_tcm/u_cpu/clk_i
  add wave /rv_soc_tcm/u_rv_tcm/u_cpu/xrst_i
  add wave /rv_soc_tcm/u_rv_tcm/u_cpu/imem_*
  add wave /rv_soc_tcm/u_rv_tcm/u_cpu/dmem_*
  add wave /rv_soc_tcm/*
  add wave /rv_soc_tcm/u_rv_tcm//*


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

  #   restart -f ;
  force -freeze  /rv_soc_tcm/xrst_i 0 0 ;
  force -freeze  /rv_soc_tcm/clk_i 1 0, 0 {10 ns} -r {20 ns} ;
  force -freeze  /rv_soc_tcm/uart_txd_i 1 0 ;
  force -freeze  /rv_soc_tcm/uart2_txd_i 1 0 ;
  force -freeze  /rv_soc_tcm/spi_miso_i  1 0 ;
  force -freeze  /rv_soc_tcm/gpio_i 00 0
  run 1 us ;
  ###   release core reset
  force -freeze  /rv_soc_tcm/xrst_i 1 0 ;
  run 1 us ;
  #   force -freeze  /rv_soc_tcm/rv_core_ena_w 1 0 ;


  #
  # write 0x80000000  0x00000013    (cpu boot address = start of tcm mem : NOP)
  # nop. nop, j 0x80000004, nop, nop. nop, nop
  #  REQ_WRITE = X"10",  LEN, ADDR3,2,1,0  DATA0,1,2,3
  REQ_WRITE 0x04 0x80000000 0x00000013 ;
  REQ_WRITE 0x04 0x80000004 0x00000013 ;
  REQ_WRITE 0x04 0x80000008 0xFFDFF06F ;
  REQ_WRITE 0x04 0x8000000C 0x00000013 ;
  REQ_WRITE 0x04 0x80000010 0x00000013 ;
  REQ_WRITE 0x04 0x80000014 0x00000013 ;
  REQ_WRITE 0x04 0x80000018 0x00000013 ;
  REQ_WRITE 0x04 0x8000001C 0x00000013 ;



  #
  # Release core reset ( write 0x000000001 to address 0xF0000000 )
  #
  #  REQ_WRITE = X"10",  LEN, ADDR3,2,1,0,  DATA0,1,2,3
  SEND_BYTE { 0x10 }
  #    LEN (bytes)
  SEND_BYTE { 0x04 }
  #    ADDR (32 bits)
  SEND_BYTE { 0xF0 }
  SEND_BYTE { 0x00 }
  SEND_BYTE { 0x00 }
  SEND_BYTE { 0x00 }
  #    DATA (LEN x bytes) 
  SEND_BYTE { 0x01 }
  SEND_BYTE { 0x00 }
  SEND_BYTE { 0x00 }
  SEND_BYTE { 0x00 }
  echo "  NOTE: Core reset released"
  run 10 us


  run 10 us ;

