#! /usr/bin/perl

#
##
### NOTE: DEFAULT MEM SIZE IS 16M words
##
#
$USAGE = "./asm2mif.pl  riscv_disassembled.txt > riscv.mif" ;

###
### NOTES:
###   - For 32 BIT instructions/data only
###   - Words can be 16-bit and misaligned but output is always 32 bits
###

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

my $asmfile  =  "";
if (($#ARGV+1) == 1)  {
    $asmfile  =  $ARGV[0];
}
else {
    print STDERR "\n  USAGE: $USAGE\n\n";
    exit;
}

if ( ! (-e $asmfile) )  {
    print STDERR "\n\n   Could not find asm file: $vecfile!\n\n";
    exit;
}

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

my $lcnt       = 0;
my $veccnt     = 0;

my @datal      = ();
my @datah      = ();
my @valids     = (); # valid addresses, i.e. addresses that have data
my ($ad_max) = 0;

my ($tmp_ad)  = 0;


open (ASM, "<$asmfile");
while (<ASM>) {
    $lcnt++;
    chop;
    #
    # ignore all debug stuff
    #
    if (/\.debug/) {
	print STDERR "\n   NOTE1: *debug* stuff ignored (l: $lcnt) and " .
	    "exiting...\n\n";
	close ASM;
	goto printout;
    }
    #
    # code ends in line like:
    #   "81f00786:   6f 00   Address 0x0000000081f00786 is out of bounds."
    #
    if (/\A\s*(\S+)\s*\:\s*\s+(\S{2})\s+(\S{2})\s+Address\s+(\S+)\s+is out of bounds\./) {
	print STDERR "\n   NOTE2:  Reached end of code on line $lcnt: \n" .
	    "           (i.e. found line with ...Address 0x00000000xxxxxxxx " .
	    "is out of bounds.)\n".
	    "           NOTE3: ALL DEBUG STUFF AFTER THAT IS IGNORED\n\n";
	close ASM;
	goto printout;
    }
    #
    # else convert to mif
    #
    if (/\A\s*8/) {
	if (m/\A\s*(\S+)\s*\:\s*(\S+)\s*/) {
	    $veccnt++;
	    $ad        = hex($1);
	    $ad        = $ad & 0x7FFFFFFF;         # reset MSB bit
            $adw       = ($ad>>2);                 # from byte to word address
	    $dat       = hex($2);
	    $dat_len   = length($2);
	    $ad_offset = $ad & 3;
	    if ( $ad_offset == 0 && $dat_len == 8 ) {            # xxxxxxxx
		$datah[$ad/4] = sprintf("%016b", $dat>>16);
		$datal[$ad/4] = sprintf("%016b", $dat&0xFFFF);
		push @valids, $ad/4;
		if ($ad_max < $ad/4) { $ad_max = $ad/4; }
	    }
	    elsif ( $ad_offset == 0 && $dat_len == 4 ) {         # ----xxxx
		$datal[$ad/4] = sprintf("%016b", $dat&0xFFFF);
		push @valids, $ad/4;
		if ($ad_max < $ad/4) { $ad_max = $ad/4; }
	    }
	    elsif ( $ad_offset == 2 && $dat_len == 4 ) {         # xxxx----
		$tmp_ad = ($ad-2)/4;
		$datah[$tmp_ad] = sprintf("%016b", $dat&0xFFFF);
		push @valids, $tmp_ad;
		if ($ad_max < $tmp_ad) { $ad_max = tmp_ad; }
	    }
	    elsif ( $ad_offset == 2 && $dat_len == 8 ) {         # xxxxxxxx
		$tmp_ad = ($ad-2)/4;
		$datah[$tmp_ad]   = sprintf("%016b", $dat&0xFFFF);
		$datal[$tmp_ad+1] = sprintf("%016b", $dat>>16);
		push @valids, $tmp_ad;
		push @valids, $tmp_ad+1;
		if ($ad_max < $tmp_ad)   { $ad_max = $tmp_ad; }
		if ($ad_max < $tmp_ad+1) { $ad_max = $tmp_ad+10;}
	    }
	    else {
		print STDERR "\nERROR1: Syntax bad on line $lcnt :\n" .
		    "         $_\n\n";
		close ASM;
		exit;          # exit without printing any data
	    }
	}
    }
}
close ASM;



printout:

###   NOTE: address starts from zero so inc by one
$ad_max = $ad_max + 1;


###
### MIF HEADER
###
print "DEPTH = $ad_max;\n";
print "WIDTH = 32;\n";
print "ADDRESS_RADIX = dec;\n";
print "DATA_RADIX = bin;\n";
print "CONTENT\n";
print "BEGIN\n";

my ($printed) = ();

foreach $ii (@valids) {
    if ($printed[$ii] ne 1 ) {
	if (defined($datah[$ii]) && defined($datal[$ii]) ) {
	    printf("%d : %s%s;\n", $ii, $datah[$ii],  $datal[$ii]);
	    $printed[$ii] = 1;
	}
	elsif ( (! defined($datah[$ii])) && defined($datal[$ii]) ) {
	    printf("%d : %s%s;\n", $ii, "0000000000000000",  $datal[$ii]);
	    $printed[$ii] = 1;
	}
	elsif ( defined($datah[$ii]) && ( ! defined($datal[$ii])) ) {
	    printf("%d : %s%s;\n", $ii, $datah[$ii], "0000000000000000");
	    $printed[$ii] = 1;
	}
	else {
	    print STDERR "\nERROR2: Error in addr and/or datah/datal\n" .
		"         $_\n\n";
	}
    }
}

print "END;\n";


print STDERR "\n   NOTE4: Found $veccnt vectors from file $asmfile\n";
print STDERR "            Max address found was: $ad_max \n\n";
